@@ -279,6 +279,10 @@ private:
 
     void drawRoundedRect(QPainter *p, const QRect &rect, bool selected, const QColor &bgcolor, bool frame, int ft, bool roundTop, bool roundBottom);
 
+    Q_REQUIRED_RESULT QColor getCategoryColor() const;
+    Q_REQUIRED_RESULT QColor getFrameColor(const QColor &resourceColor, const QColor &categoryColor) const;
+    Q_REQUIRED_RESULT QColor getBackgroundColor(const QColor &resourceColor, const QColor &categoryColor) const;
+
     int mCellXLeft, mCellXRight;
     int mCellYTop, mCellYBottom;
 
